/*
** configrw.c     
**
** Various IOCTLs whose IRPs are always completed without ever
** being made pending are serviced by the functions in this module.
*/

#include "platform.h"
#include "ioctl.h"
#include "device.h"
#include "eeprom.h"
#include "configrw.h"

OS_STATUS
ADMXRC2_ReadConfig(
  ADMXRC*        devCtx,
  OS_IO_REQUEST* req)
{
  ADMXRC2_IOCTLS_READCONFIG* iptr;
  ADMXRC2_IOCTLS_READCONFIG* optr;
  unsigned long              index, val, size;

  iptr = req->param.inBuf;
  if (iptr == NULL || req->param.inSize != sizeof(iptr->in)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  optr = req->param.outBuf;
  if (optr == NULL || req->param.outSize != sizeof(optr->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  index = iptr->in.index;

  switch (devCtx->info.eepromType) {
  case ADMXRC_EEPROM_9346:
    size = 0x40;
    break;
  case ADMXRC_EEPROM_9356:
    size = 0x80;
    break;
  case ADMXRC_EEPROM_9366:
    size = 0x100;
    break;
  default:
    return OS_MAKE_STATUS(ADMXRC2_NOT_SUPPORTED);
  }

  if (index >= size) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  ADMXRC_ReadEprom(devCtx, index, &val);
  optr->out.value = val;

  req->status.length = sizeof(optr->out);

  return OS_STATUS_SUCCESS;
}

OS_STATUS
ADMXRC2_WriteConfig(
  ADMXRC*        devCtx,
  OS_IO_REQUEST* req)
{
  ADMXRC2_IOCTLS_WRITECONFIG* iptr;
  unsigned long               size, index, val;

  iptr = req->param.inBuf;
  if (iptr == NULL || req->param.inSize != sizeof(iptr->in)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  if (iptr->in.signature1 != 0x587d3f31 ||
      iptr->in.signature2 != 0x043a749f) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  index = iptr->in.index;
  val = iptr->in.value;

  switch (devCtx->info.eepromType) {
  case ADMXRC_EEPROM_9346:
    size = 0x40;
    break;
  case ADMXRC_EEPROM_9356:
    size = 0x80;
    break;
  case ADMXRC_EEPROM_9366:
    size = 0x100;
    break;
  default:
    return OS_MAKE_STATUS(ADMXRC2_NOT_SUPPORTED);
  }

  if (index >= size) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  ADMXRC_WriteEprom(devCtx, index, val);

  return OS_STATUS_SUCCESS;
}

