/*
** owner.c
**
** IOCTL handlers for taking/releasing ownership of a card
*/

#include "platform.h"
#include "ioctl.h"
#include "device.h"
#include "owner.h"

static OS_STATUS
grabCard(
  ADMXRC*        devCtx,
  OS_USER_CLIENT handle,
  unsigned long  cardid)
{
  OS_STATUS         status = OS_STATUS_SUCCESS;
  OS_SPINLOCK_FLAGS f;

  os_acquire_spinlock(&devCtx->owner.lock, &f);
  if (devCtx->owner.handle != NULL) {
    os_release_spinlock(&devCtx->owner.lock, f);
    status = OS_MAKE_STATUS(ADMXRC2_CARD_BUSY);
    os_dbgprint(3, ("grabCard: card %ld busy\n",
		    (unsigned long) devCtx->info.card.CardID));
  } else {
    if (cardid == 0 || cardid == devCtx->info.card.CardID) {
      devCtx->owner.handle = handle;
      os_release_spinlock(&devCtx->owner.lock, f);
      os_dbgprint(3, ("grabCard: card %ld grabbed, owner = 0x%p\n",
		      (unsigned long) devCtx->info.card.CardID, handle));
    } else {
      os_release_spinlock(&devCtx->owner.lock, f);
      os_dbgprint(3, ("grabCard: this card's ID (%ld) is not %ld\n",
		      (unsigned long) devCtx->info.card.CardID,		      
		      (unsigned long) cardid));
      status = OS_MAKE_STATUS(ADMXRC2_CARD_NOT_FOUND);
    }
  }

  return status;
}

OS_STATUS
ADMXRC_OpenCard(
  ADMXRC*        devCtx,
  OS_USER_CLIENT handle,
  OS_IO_REQUEST* req)
{
  ADMXRC_IOCTLS_OPENCARD* iptr;
  OS_STATUS               status;
  unsigned long           card_id;

  iptr = (ADMXRC_IOCTLS_OPENCARD*) req->param.inBuf;
  if (iptr == NULL || req->param.inSize != sizeof(iptr->in)) {
    status = OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  card_id = iptr->in.card_id;
  if (devCtx->info.card.BoardType == ADMXRC2_BOARD_ADMXRC ||
      devCtx->info.card.BoardType == ADMXRC2_BOARD_ADMXRC_P) {
    status = grabCard(devCtx, handle, card_id);
  } else {
    status = OS_MAKE_STATUS(ADMXRC2_CARD_ERROR);
    os_dbgprint(1, ("ADMXRC_OpenCard: card %ld is not ADM-XRC/ADM-XRC-P\n",
		    (unsigned long) card_id));
  }
  
  return status;
}
		
OS_STATUS
ADMXRC2_OpenCard(
  ADMXRC*        devCtx,
  OS_USER_CLIENT handle,
  OS_IO_REQUEST* req)
{
  ADMXRC2_IOCTLS_OPENCARD* iptr;
  OS_STATUS                status;
			
  iptr = (ADMXRC2_IOCTLS_OPENCARD*) req->param.inBuf;
  if (iptr == NULL || req->param.inSize != sizeof(iptr->in)) {
    status = OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  status = grabCard(devCtx, handle, iptr->in.card_id);

  return status;
}

OS_STATUS
ADMXRC2_OpenCardByIndex(
  ADMXRC*        devCtx,
  OS_USER_CLIENT handle,
  OS_IO_REQUEST* req)
{
  return grabCard(devCtx, handle, 0);
}

void
ADMXRC_CloseCard(
  ADMXRC*        devCtx,
  OS_USER_CLIENT handle)
{
  OS_SPINLOCK_FLAGS f;

  os_acquire_spinlock(&devCtx->owner.lock, &f);
  if (devCtx->owner.handle == handle) {
    devCtx->owner.handle = NULL;
  }
  os_release_spinlock(&devCtx->owner.lock, f);
}
