/*
** struct.h - structures and types used in ADM-XRC API calls
**
** (C) Copyright 2000 Alpha Data Parallel Systems Ltd.
*/

#ifndef _ADATA_ADMXRC2_STRUCT_H
#define _ADATA_ADMXRC2_STRUCT_H

#if !defined(__KERNEL__)
#   include <stdint.h>
#endif

/*
** Handle to a card
*/
typedef int ADMXRC2_HANDLE;

/*
** Handle to an Event
*/
typedef int ADMXRC2_EVENT;

/*
** A type that holds a Card ID
*/
typedef unsigned long ADMXRC2_CARDID;

/*
** Card enumeration function type
*/
typedef void (*ADMXRC2_ENUM_FUNCTION)(unsigned int index, ADMXRC2_HANDLE card);

/*
** Handle to a DMA descriptor
*/
typedef unsigned long ADMXRC2_DMADESC;

/*
** FPGA bitstream image
*/
typedef void* ADMXRC2_IMAGE;

/*
** Error handler function type
*/
typedef void (*ADMXRC2_ERROR_HANDLER)(const char* FnName, ADMXRC2_STATUS Status);

/*
** DMA direction options
*/
typedef enum _ADMXRC2_DMADIR
{
    ADMXRC2_PCITOLOCAL = 0,
    ADMXRC2_LOCALTOPCI = 1
} ADMXRC2_DMADIR;

/*
** Bus width value
*/
typedef enum _ADMXRC2_IOWIDTH
{
    ADMXRC2_IOWIDTH_8      = 0,
    ADMXRC2_IOWIDTH_16     = 1,
    ADMXRC2_IOWIDTH_32     = 2,
    ADMXRC2_IOWIDTH_64     = 3,
    ADMXRC2_IOWIDTH_32BITS = 0x7FFFFFFF
} ADMXRC2_IOWIDTH;

/*
** Misc DMA mode options for ADMXRC2_BuildDmaModeWord()
*/
#define ADMXRC2_DMAMODE_IGNOREREADY     (0x0 << 6)      /* default */
#define ADMXRC2_DMAMODE_USEREADY        (0x1 << 6)
#define ADMXRC2_DMAMODE_IGNOREBTERM     (0x0 << 7)      /* default */
#define ADMXRC2_DMAMODE_USEBTERM        (0x1 << 7)
#define ADMXRC2_DMAMODE_BURSTDISABLE    (0x0 << 8)      /* default */
#define ADMXRC2_DMAMODE_BURSTENABLE     (0x1 << 8)
#define ADMXRC2_DMAMODE_INCREMENTLOCAL  (0x0 << 11)     /* default */
#define ADMXRC2_DMAMODE_FIXEDLOCAL      (0x1 << 11)
#define ADMXRC2_DMAMODE_NONDEMAND       (0x0 << 12)     /* default */
#define ADMXRC2_DMAMODE_DEMAND          (0x1 << 12)
#define ADMXRC2_DMAMODE_IGNOREEOT       (0x0 << 14)     /* default */
#define ADMXRC2_DMAMODE_USEEOT          (0x1 << 14)

/*
** This special value for a DMA channel allows a DMA transfer
** to take place on the first free DMA channel.
*/
#define ADMXRC2_DMACHAN_ANY (0xFFU)

/*
** ADMXRC2_DoDMA() flags
*/
#define ADMXRC2_DMAFLAG_CANQUEUE	(0x0 << 0)      /* default */
#define ADMXRC2_DMAFLAG_DONOTQUEUE	(0x1 << 0)
#define ADMXRC2_DMAFLAG_TIMEOUT         (2)

/*
** Flags for ADMXRC2_Read and ADMXRC2_Write
*/
#define ADMXRC2_IOFIXED	  (1 << 0)
#define ADMXRC2_IOLOCAL	  (0 << 0) /* default */
#define ADMXRC2_IOADAPTER (1 << 1)

/*
** FPGA Type
*/
typedef enum _ADMXRC2_FPGA_TYPE
{
    ADMXRC2_FPGA_RESVD0      = 0,
    ADMXRC2_FPGA_RESVD1      = 1,
    ADMXRC2_FPGA_RESVD2      = 2,
    ADMXRC2_FPGA_RESVD3      = 3,
    ADMXRC2_FPGA_V1000       = 4,
    ADMXRC2_FPGA_V400        = 5,
    ADMXRC2_FPGA_V600        = 6,
    ADMXRC2_FPGA_V800        = 7,
    ADMXRC2_FPGA_V2000E      = 8,
    ADMXRC2_FPGA_V1000E      = 9,
    ADMXRC2_FPGA_V1600E      = 10,
    ADMXRC2_FPGA_V3200E      = 11,
    ADMXRC2_FPGA_V812E       = 12,
    ADMXRC2_FPGA_V405E       = 13,
    ADMXRC2_FPGA_RESVD14     = 14,
    ADMXRC2_FPGA_RESVD15     = 15,
    ADMXRC2_FPGA_RESVD16     = 16,
    ADMXRC2_FPGA_RESVD17     = 17,
    ADMXRC2_FPGA_RESVD18     = 18,
    ADMXRC2_FPGA_RESVD19     = 19,
    ADMXRC2_FPGA_RESVD20     = 20,
    ADMXRC2_FPGA_RESVD21     = 21,
    ADMXRC2_FPGA_RESVD22     = 22,
    ADMXRC2_FPGA_RESVD23     = 23,
    ADMXRC2_FPGA_RESVD24     = 24,
    ADMXRC2_FPGA_RESVD25     = 25,
    ADMXRC2_FPGA_RESVD26     = 26,
    ADMXRC2_FPGA_RESVD27     = 27,
    ADMXRC2_FPGA_RESVD28     = 28,
    ADMXRC2_FPGA_RESVD29     = 29,
    ADMXRC2_FPGA_RESVD30     = 30,
    ADMXRC2_FPGA_RESVD31     = 31,
    ADMXRC2_FPGA_2V1000      = 32,
    ADMXRC2_FPGA_2V1500      = 33,
    ADMXRC2_FPGA_2V2000      = 34,
    ADMXRC2_FPGA_2V3000      = 35,
    ADMXRC2_FPGA_2V4000      = 36,
    ADMXRC2_FPGA_2V6000      = 37,
    ADMXRC2_FPGA_2V8000      = 38,
    ADMXRC2_FPGA_2V10000     = 39,
    ADMXRC2_FPGA_UNKNOWN     = 40,
    ADMXRC2_FPGA_TYPE_32BITS = 0x7FFFFFFF
} ADMXRC2_FPGA_TYPE;

/*
** In this interface, programmable clocks are referred to by
** an index.
**
** Index 0 always corresponds to the local bus clock
*/
#define ADMXRC2_CLOCK_LCLK	(0)

/*
** ADM-XRC board types, used in ADMXRC2_CARD_INFO2::BoardType
*/
typedef enum _ADMXRC2_BOARD_TYPE
{
    ADMXRC2_BOARD_ADMXRC            = 0,    /* ADM-XRC         */
    ADMXRC2_BOARD_ADMXRC_P          = 1,    /* ADM-XRC-P       */
    ADMXRC2_BOARD_ADMXRC2_LITE      = 2,    /* ADM-XRC-II-Lite */
    ADMXRC2_BOARD_ADMXRC2           = 3,    /* ADM-XRC-II      */
    ADMXRC2_BOARD_ADMXRC2_PRO       = 4,    /* ADM-XRC-II-Pro  */
    ADMXRC2_BOARD_UNKNOWN           = 5,
    ADMXRC2_BOARD_TYPE_32BITS       = 0x7FFFFFFF
} ADMXRC2_BOARD_TYPE;

/*
** Card information structure
*/
typedef struct _ADMXRC2_CARD_INFO
{
    ADMXRC2_CARDID           CardID;           /* ID of card                               */
    uint32_t                 SerialNum;        /* serial number of card                    */
    ADMXRC2_BOARD_TYPE       BoardType;        /* type of card                             */
    ADMXRC2_FPGA_TYPE        FPGAType;         /* type of FPGA fitted                      */
    unsigned long            NumClock;         /* number of programmable clocks            */
    unsigned long            NumDMAChan;       /* number of DMA channels                   */
    unsigned long            NumRAMBank;       /* number of RAM banks (fitted or not)      */
    unsigned long            NumSpace;         /* number of spaces                         */
    uint32_t                 RAMBanksFitted;   /* bitmap of banks fitted (max 32 banks)    */
    uint8_t                  BoardRevision;    /* board rev, 0xab, a = major, b = minor    */
    uint8_t                  LogicRevision;    /* logic rev, 0xab, a = major, b = minor    */
} ADMXRC2_CARD_INFO;

/*
** RAM type flags, used in ADMXRC2_BANK_INFO::Type bitmask
*/
#define ADMXRC2_RAM_ZBTFT    (1 << 0)    /* ZBT flowthrough */
#define ADMXRC2_RAM_ZBTP     (1 << 1)    /* ZBT pipelined   */

/*
** RAM bank information structure
*/
typedef struct _ADMXRC2_BANK_INFO
{
    unsigned long      Type;      /* bitmask of RAM mode(s) supported      */
    unsigned long      Width;     /* in bits, valid whether fitted or not  */
    unsigned long      Size;      /* in words, valid whether fitted or not */
    int                Fitted;    /* TRUE if present (fitted to card)      */
} ADMXRC2_BANK_INFO;

/*
** Structure returning version information
*/
typedef struct _ADMXRC2_VERSION_INFO
{
    uint8_t DriverMinor;
    uint8_t DriverMajor;
    uint8_t APIMinor;
    uint8_t APIMajor;
} ADMXRC2_VERSION_INFO;

/*
** A structure used to map a user buffer for Direct Master
** access by the FPGA.
*/
typedef struct _ADMXRC2_BUFFERMAP
{
    unsigned long  MaxPages;        /* User initialises to length of PCI address buffer      */
    uint32_t*      PagesPci;        /* User initialises to point to PCI address buffer       */
    unsigned long  PageLength;      /* Receives size in bytes of a page                      */
    unsigned long  PageBits;        /* Number of bits representing a page offset             */
    unsigned long  PagesSpanned;    /* Receives number of pages spanned                      */
    unsigned long  BytesSpanned;    /* Receives number of bytes spanned                      */
    unsigned long  InitOffset;      /* Receives page offset within 1st page of start of span */
} ADMXRC2_BUFFERMAP;

/*
** Type of syncing to be done by ADMXRC2_SyncDirectMaster()
*/
typedef enum _ADMXRC2_SYNCMODE
{
    ADMXRC2_SYNC_CPUTOFPGA = 0x1,    /* CPU has set up buffer for FPGA to access */
    ADMXRC2_SYNC_FPGATOCPU = 0x2     /* FPGA has modified buffer                 */
} ADMXRC2_SYNCMODE;

typedef struct _ADMXRC2_SPACE_INFO
{
    void*         VirtualBase;      /* Application space window         */
    unsigned long VirtualSize;	    /* Size of application space window	*/
    unsigned long PhysicalBase;	    /* Host I/O bus address             */
    unsigned long LocalBase;        /* Local bus address                */
    unsigned long LocalSize;        /* Size in bytes of local bus space	*/
} ADMXRC2_SPACE_INFO;

#endif

